@@ -41,8 +41,8 @@ use constant WRITE_ONLY     => sub { $_[0] ? 'wb' . shift : 'w' };
 use constant MODE_READ      => sub { $_[0] =~ /^r/ ? 1 : 0 };
 
 # Pointless assigment to make -w shut up
-my $getpwuid = 'unknown' unless eval { my $f = getpwuid (0); };
-my $getgrgid = 'unknown' unless eval { my $f = getgrgid (0); };
+my $getpwuid; $getpwuid = 'unknown' unless eval { my $f = getpwuid (0); };
+my $getgrgid; $getgrgid = 'unknown' unless eval { my $f = getgrgid (0); };
 use constant UNAME          => sub { $getpwuid || scalar getpwuid( shift() ) };
 use constant GNAME          => sub { $getgrgid || scalar getgrgid( shift() ) };
 use constant UID            => $>;
